﻿using System;
using Windows.ApplicationModel.Background;
using Windows.Storage;
using Windows.System.Threading;

namespace TestLibrary
{
    /// <summary>
    /// 샘플 백그라운드 태스크 1
    /// </summary>
    public sealed class SampleBackgroundTask1 : IBackgroundTask
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 취소 사유
        /// </summary>
        private BackgroundTaskCancellationReason cancellationReason = BackgroundTaskCancellationReason.Abort;

        /// <summary>
        /// 취소 요청 여부
        /// </summary>
        private volatile bool cancelRequested = false;

        /// <summary>
        /// 백그라운드 태스크 지연
        /// </summary>
        private BackgroundTaskDeferral deferral = null;

        /// <summary>
        /// 스레드 풀 타이머
        /// </summary>
        private ThreadPoolTimer threadPoolTimer = null;

        /// <summary>
        /// 진행률
        /// </summary>
        private uint progress = 0;

        /// <summary>
        /// 백그라운드 태스크 인스턴스
        /// </summary>
        private IBackgroundTaskInstance taskInstance = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 실행하기 - Run(taskInstance)

        /// <summary>
        /// 실행하기
        /// </summary>
        /// <param name="taskInstance">백그라운드 태스크 인스턴스</param>
        public void Run(IBackgroundTaskInstance taskInstance)
        {
            BackgroundWorkCostValue costValue = BackgroundWorkCost.CurrentBackgroundWorkCost;

            ApplicationDataContainer container = ApplicationData.Current.LocalSettings;

            container.Values["BackgroundWorkCost"] = costValue.ToString();

            taskInstance.Canceled += taskInstance_Canceled;

            this.deferral = taskInstance.GetDeferral();

            this.taskInstance = taskInstance;

            this.threadPoolTimer = ThreadPoolTimer.CreatePeriodicTimer
            (
                new TimerElapsedHandler(TimerElapsedCallback),
                TimeSpan.FromSeconds(1)
            );
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 백그라운드 태스크 인스턴스 취소시 처리하기 - taskInstance_Canceled(sender, e)

        /// <summary>
        /// 백그라운드 태스크 인스턴스 취소시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void taskInstance_Canceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason e)
        {
            this.cancelRequested = true;

            this.cancellationReason = e;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 타이머 경과시 콜백 처리하기 - TimerElapsedCallback(timer)

        /// <summary>
        /// 타이머 경과시 콜백 처리하기
        /// </summary>
        /// <param name="timer">스레드 풀 타이머</param>
        private void TimerElapsedCallback(ThreadPoolTimer timer)
        {
            if((this.cancelRequested == false) && (this.progress < 100))
            {
                this.progress += 10;

                this.taskInstance.Progress = this.progress;
            }
            else
            {
                this.threadPoolTimer.Cancel();

                string key = this.taskInstance.Task.Name;

                string taskStatus = (this.progress < 100) ? "취소 : " + this.cancellationReason.ToString() : "완료";

                ApplicationDataContainer container = ApplicationData.Current.LocalSettings;

                container.Values[key] = taskStatus;

                this.deferral.Complete();
            }
        }

        #endregion
    }
}