﻿using Windows.ApplicationModel.Background;
using Windows.Storage;

namespace TestLibrary
{
    /// <summary>
    /// 샘플 백그라운드 태스크 2
    /// </summary>
    public sealed class SampleBackgroundTask2 : IBackgroundTask
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 취소 요청 여부
        /// </summary>
        private volatile bool cancelRequested = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 실행하기 - Run(taskInstance)

        /// <summary>
        /// 실행하기
        /// </summary>
        /// <param name="taskInstance">태스크 인스턴스</param>
        public void Run(IBackgroundTaskInstance taskInstance)
        {
            taskInstance.Canceled += taskInstance_Canceled;

            uint Progress;

            for(Progress = 0; Progress <= 100; Progress += 10)
            {
                if(this.cancelRequested)
                {
                    break;
                }

                taskInstance.Progress = Progress;
            }

            ApplicationDataContainer container = ApplicationData.Current.LocalSettings;

            string key = taskInstance.Task.Name;

            container.Values[key] = (Progress < 100) ? "취소" : "완료";
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 태스크 인스턴스 취소시 처리하기 - taskInstance_Canceled(sender, e)

        /// <summary>
        /// 태스크 인스턴스 취소시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void taskInstance_Canceled(IBackgroundTaskInstance sender, BackgroundTaskCancellationReason e)
        {
            this.cancelRequested = true;
        }

        #endregion
    }
}