﻿using System;
using System.Collections.Generic;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.UI;
using Windows.UI.Core;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Automation.Peers;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 현재 페이지
        /// </summary>
        public static MainPage CurrentPage;

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 시나리오 리스트
        /// </summary>
        private List<Scenario> scenarioList = new List<Scenario>
        {
            new Scenario() { Title = "백그라운드 태스크"                    , ClassType = typeof(SamplePage1) },
            new Scenario() { Title = "조건부 백그라운드 태스크"             , ClassType = typeof(SamplePage2) },
            new Scenario() { Title = "서비스 완료 태스크"                   , ClassType = typeof(SamplePage3) },
            new Scenario() { Title = "시간 트리거 백그라운드 태스크"        , ClassType = typeof(SamplePage4) },
            new Scenario() { Title = "애플리케이션 트리거 백그라운드 태스크", ClassType = typeof(SamplePage5) },
            new Scenario() { Title = "그룹 백그라운드 태스크"               , ClassType = typeof(SamplePage6) }
        };

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "백그라운드 태스크 사용하기";

            #endregion

            CurrentPage = this;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            List<Scenario> sourceList = new List<Scenario>();

            int i = 1;

            foreach(Scenario scenario in scenarioList)
            {
                sourceList.Add
                (
                    new Scenario
                    {
                        Title     = $"{i++}) {scenario.Title}",
                        ClassType = scenario.ClassType
                    }
                );
            }

            this.listBox.ItemsSource = sourceList;

            if(Window.Current.Bounds.Width < 640)
            {
                this.listBox.SelectedIndex = -1;
            }
            else
            {
                this.listBox.SelectedIndex = 0;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 창 토글 버튼 클릭시 처리하기 - paneToggleButton_Click(sender, e)

        /// <summary>
        /// 창 토글 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void paneToggleButton_Click(object sender, RoutedEventArgs e)
        {
            this.splitView.IsPaneOpen = !splitView.IsPaneOpen;
        }

        #endregion
        #region 리스트 박스 선택 변경시 처리하기 - listBox_SelectionChanged(sender, e)

        /// <summary>
        /// 리스트 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ShowStatusMessage(string.Empty, NotifyType.StatusMessage);

            ListBox listBox = sender as ListBox;

            Scenario scenario = listBox.SelectedItem as Scenario;

            if(scenario != null)
            {
                this.frame.Navigate(scenario.ClassType);

                if(Window.Current.Bounds.Width < 640)
                {
                    this.splitView.IsPaneOpen = false;
                }
            }
        }

        #endregion
        #region 상태 메시지 업데이트하기 - UpdateStatusMessage(message, notifyType)

        /// <summary>
        /// 상태 메시지 업데이트하기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <param name="notifyType">통지 타입</param>
        private void UpdateStatusMessage(string message, NotifyType notifyType)
        {
            switch(notifyType)
            {
                case NotifyType.StatusMessage :

                    this.statusBorder.Background = new SolidColorBrush(Colors.Green);

                    break;

                case NotifyType.ErrorMessage :

                    this.statusBorder.Background = new SolidColorBrush(Colors.Red);

                    break;
            }

            this.statusTextBlock.Text = message;

            this.statusBorder.Visibility = (this.statusTextBlock.Text != string.Empty) ? Visibility.Visible : Visibility.Collapsed;

            if(this.statusTextBlock.Text != string.Empty)
            {
                this.statusBorder.Visibility     = Visibility.Visible;
                this.statusStackPanel.Visibility = Visibility.Visible;
            }
            else
            {
                this.statusBorder.Visibility     = Visibility.Collapsed;
                this.statusStackPanel.Visibility = Visibility.Collapsed;
            }

            AutomationPeer peer = FrameworkElementAutomationPeer.FromElement(this.statusTextBlock);

            if(peer != null)
            {
                peer.RaiseAutomationEvent(AutomationEvents.LiveRegionChanged);
            }
        }

        #endregion
        #region 상태 메시지 표시하기 - ShowStatusMessage(message, notifyType)

        /// <summary>
        /// 상태 메시지 표시하기
        /// </summary>
        /// <param name="message">메시지</param>
        /// <param name="notifyType">통지 타입</param>
        public async void ShowStatusMessage(string message, NotifyType notifyType)
        {
            if(Dispatcher.HasThreadAccess)
            {
                UpdateStatusMessage(message, notifyType);
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () => UpdateStatusMessage(message, notifyType));
            }
        }

        #endregion
    }
}