﻿using System;
using System.Collections.Generic;
using Windows.ApplicationModel.Background;
using Windows.Foundation;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 2
    /// </summary>
    public sealed partial class SamplePage2 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage2()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage2()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            foreach(KeyValuePair<Guid, IBackgroundTaskRegistration> keyValuePair in BackgroundTaskRegistration.AllTasks)
            {
                if(keyValuePair.Value.Name == BackgroundTaskHelper.BackgroundTaskName2)
                {
                    AttachEventHandler(keyValuePair.Value);

                    BackgroundTaskHelper.UpdateBackgroundTaskRegistrationStatus(BackgroundTaskHelper.BackgroundTaskName2, true);

                    break;
                }
            }

            UpdateUI();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 등록 버튼 클릭시 처리하기 - registerButton_Click(sender, e)

        /// <summary>
        /// 등록 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void registerButton_Click(object sender, RoutedEventArgs e)
        {
            BackgroundTaskRegistration backgroundTaskRegistration = BackgroundTaskHelper.RegisterBackgroundTask
            (
                BackgroundTaskHelper.BackgroundTaskEntryPoint2,
                BackgroundTaskHelper.BackgroundTaskName2,
                new SystemTrigger(SystemTriggerType.TimeZoneChange, false),
                new SystemCondition(SystemConditionType.InternetAvailable)
            );

            AttachEventHandler(backgroundTaskRegistration);

            UpdateUI();
        }

        #endregion
        #region 등록 취소 버튼 클릭시 처리하기 - unregisterButton_Click(sender, e)

        /// <summary>
        /// 등록 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void unregisterButton_Click(object sender, RoutedEventArgs e)
        {
            BackgroundTaskHelper.UnregisterBackgroundTasks(BackgroundTaskHelper.BackgroundTaskName2);

            UpdateUI();
        }

        #endregion

        #region 백그라운드 태스크 등록 진행시 처리하기 - backgroundTaskRegistration_Progress(sender, e)

        /// <summary>
        /// 백그라운드 태스크 등록 진행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void backgroundTaskRegistration_Progress(IBackgroundTaskRegistration sender, BackgroundTaskProgressEventArgs e)
        {
            IAsyncAction action = Dispatcher.RunAsync
            (
                CoreDispatcherPriority.Normal,
                () =>
                {
                    string message = $"진행률 : {e.Progress}%";
                
                    BackgroundTaskHelper.BackgroundTaskProgress2 = message;
                
                    UpdateUI();
                }
            );
        }

        #endregion
        #region 백그라운드 태스크 등록 완료시 처리하기 - backgroundTaskRegistration_Completed(sender, e)

        /// <summary>
        /// 백그라운드 태스크 등록 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void backgroundTaskRegistration_Completed(IBackgroundTaskRegistration sender, BackgroundTaskCompletedEventArgs e)
        {
            UpdateUI();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 이벤트 핸들러 부착하기 - AttachEventHandler(backgroundTaskRegistration)

        /// <summary>
        /// 이벤트 핸들러 부착하기
        /// </summary>
        /// <param name="backgroundTaskRegistration">백그라운드 태스크 등록</param>
        private void AttachEventHandler(IBackgroundTaskRegistration backgroundTaskRegistration)
        {
            backgroundTaskRegistration.Progress  += backgroundTaskRegistration_Progress;
            backgroundTaskRegistration.Completed += backgroundTaskRegistration_Completed;
        }

        #endregion
        #region UI 업데이트하기 - UpdateUI()

        /// <summary>
        /// UI 업데이트하기
        /// </summary>
        private async void UpdateUI()
        {
            await Dispatcher.RunAsync
            (
                CoreDispatcherPriority.Normal,
                () =>
                {
                    this.registerButton.IsEnabled   = !BackgroundTaskHelper.BackgroundTaskRegistered2;
                    this.unregisterButton.IsEnabled =  BackgroundTaskHelper.BackgroundTaskRegistered2;

                    this.statusTextBlock.Text   = BackgroundTaskHelper.GetBackgroundTaskStatus(BackgroundTaskHelper.BackgroundTaskName2);
                    this.progressTextBlock.Text = BackgroundTaskHelper.BackgroundTaskProgress2;
                }
            );
        }

        #endregion
    }
}