﻿using System;
using System.Collections.Generic;
using Windows.ApplicationModel.Background;
using Windows.Foundation;
using Windows.Storage;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;

namespace TestProject
{
    /// <summary>
    /// 샘플 페이지 5
    /// </summary>
    public sealed partial class SamplePage5 : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 애플리케이션 트리거
        /// </summary>
        private ApplicationTrigger trigger = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - SamplePage5()

        /// <summary>
        /// 생성자
        /// </summary>
        public SamplePage5()
        {
            InitializeComponent();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 탐색되는 경우 처리하기 - OnNavigatedTo(e)

        /// <summary>
        /// 탐색되는 경우 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            foreach(KeyValuePair<Guid, IBackgroundTaskRegistration> keyValuePair in BackgroundTaskRegistration.AllTasks)
            {
                if(keyValuePair.Value.Name == BackgroundTaskHelper.BackgroundTaskName5)
                {
                    AttachEventHandler(keyValuePair.Value);

                    BackgroundTaskHelper.UpdateBackgroundTaskRegistrationStatus(BackgroundTaskHelper.BackgroundTaskName5, true);

                    break;
                }
            }

            this.trigger = new ApplicationTrigger();

            UpdateUI();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 등록 버튼 클릭시 처리하기 - registerButton_Click(sender, e)

        /// <summary>
        /// 등록 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void registerButton_Click(object sender, RoutedEventArgs e)
        {
            BackgroundTaskRegistration backgroundTaskRegistration = BackgroundTaskHelper.RegisterBackgroundTask
            (
                BackgroundTaskHelper.BackgroundTaskEntryPoint5,
                BackgroundTaskHelper.BackgroundTaskName5,
                this.trigger,
                null
            );

            AttachEventHandler(backgroundTaskRegistration);

            UpdateUI();
        }

        #endregion
        #region 등록 취소 버튼 클릭시 처리하기 - unregisterButton_Click(sender, e)

        /// <summary>
        /// 등록 취소 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void unregisterButton_Click(object sender, RoutedEventArgs e)
        {
            BackgroundTaskHelper.UnregisterBackgroundTasks(BackgroundTaskHelper.BackgroundTaskName5);

            BackgroundTaskHelper.BackgroundTaskResult5 = string.Empty;

            UpdateUI();
        }

        #endregion
        #region 신호 버튼 클릭시 처리하기 - signalButton_Click(sender, e)

        /// <summary>
        /// 신호 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private async void signalButton_Click(object sender, RoutedEventArgs e)
        {
            ApplicationDataContainer container = ApplicationData.Current.LocalSettings;

            container.Values.Remove(BackgroundTaskHelper.BackgroundTaskName5);

            ApplicationTriggerResult result = await this.trigger.RequestAsync();

            BackgroundTaskHelper.BackgroundTaskResult5 = $"신호 결과 : {result.ToString()}";

            UpdateUI();
        }

        #endregion

        #region 백그라운드 태스크 등록 진행시 처리하기 - backgroundTaskRegistration_Progress(sender, e)

        /// <summary>
        /// 백그라운드 태스크 등록 진행시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void backgroundTaskRegistration_Progress(IBackgroundTaskRegistration sender, BackgroundTaskProgressEventArgs e)
        {
            IAsyncAction action = Dispatcher.RunAsync
            (
                CoreDispatcherPriority.Normal,
                () =>
                {
                    string message = $"진행률 : {e.Progress}%";

                    BackgroundTaskHelper.BackgroundTaskProgress5 = message;

                    UpdateUI();
                }
            );
        }

        #endregion
        #region 백그라운드 태스크 등록 완료시 처리하기 - backgroundTaskRegistration_Completed(sender, e)

        /// <summary>
        /// 백그라운드 태스크 등록 완료시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void backgroundTaskRegistration_Completed(IBackgroundTaskRegistration sender, BackgroundTaskCompletedEventArgs e)
        {
            UpdateUI();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 이벤트 핸들러 부착하기 - AttachEventHandler(backgroundTaskRegistration)

        /// <summary>
        /// 이벤트 핸들러 부착하기
        /// </summary>
        /// <param name="backgroundTaskRegistration">백그라운드 태스크 등록</param>
        private void AttachEventHandler(IBackgroundTaskRegistration backgroundTaskRegistration)
        {
            backgroundTaskRegistration.Progress  += backgroundTaskRegistration_Progress;
            backgroundTaskRegistration.Completed += backgroundTaskRegistration_Completed;
        }

        #endregion
        #region UI 업데이트하기 - UpdateUI()

        /// <summary>
        /// UI 업데이트하기
        /// </summary>
        private async void UpdateUI()
        {
            await Dispatcher.RunAsync
            (
                CoreDispatcherPriority.Normal,
                () =>
                {
                    this.registerButton.IsEnabled   = !BackgroundTaskHelper.BackgroundTaskRegistered5;
                    this.unregisterButton.IsEnabled =  BackgroundTaskHelper.BackgroundTaskRegistered5;
                    this.signalButton.IsEnabled     =  BackgroundTaskHelper.BackgroundTaskRegistered5 & (this.trigger != null);

                    this.progressTextBlock.Text = BackgroundTaskHelper.BackgroundTaskProgress5;
                    this.statusTextBlock.Text   = BackgroundTaskHelper.GetBackgroundTaskStatus(BackgroundTaskHelper.BackgroundTaskName5);
                    this.resultTextBlock.Text   = BackgroundTaskHelper.BackgroundTaskResult5;
                }
            );
        }

        #endregion
    }
}